/*!
 * (C) Emu Analytics
 */
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
  var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
  if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
  else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
  return c > 3 && r && Object.defineProperty(target, key, r), r;
};
import { Component, Element, Host, h, Prop, Watch } from '@stencil/core';
import { AncestorElement } from '../../../utils/component.utils';
import { randomId } from '../../../utils/random-id';
import { debugLifecycle, formatEl } from '../../../utils/debug';
import { FlowCoreMapLayer } from '../../../core/flow-core-map/flow-core-map-layer';
/**
 * @sing 1.0
 * @status stable
 * @reactive
 *
 * @slot - The layer definition.
 */
export class FlowMapLayer {
  constructor() {
    /**
     * Set to `true` to enable hover and click actions on the map layer.
     */
    this.interactive = true;
    /** The minimum zoom level for the layer. Below this level, the layer will be hidden. */
    this.minZoom = 0;
    /** The maximum zoom level for the layer. Above this level, the layer will be hidden. */
    this.maxZoom = 18;
  }
  updateInteractive(val) {
    this._layer.options.interactive = val;
  }
  updateMinZoom(val) {
    this._layer.minZoom = val;
  }
  updateMaxZoom(val) {
    this._layer.maxZoom = val;
  }
  async componentWillLoad() {
    var _a;
    if (!this.el.id) {
      this.el.setAttribute('id', (_a = this.name) !== null && _a !== void 0 ? _a : randomId('layer'));
    }
    debugLifecycle(`${formatEl(this.el)} will load`);
    await this.flowMap._map.initializeContext();
    this._layer = new FlowCoreMapLayer({
      id: this.el.id,
      flowMap: this.flowMap._map,
      type: this.type,
      source: this.source,
      sourceLayer: this.sourceLayer,
      interactive: this.interactive,
      above: this.above,
      below: this.below,
      minZoom: this.minZoom,
      maxZoom: this.maxZoom
    });
    this.setFilter(this.filter);
    this.setStyle(this.layerStyle);
    // Add mutation observer to watch changes to text node
    this.mutationObserver = new MutationObserver(ev => this.setStyle(this.layerStyle));
    this.mutationObserver.observe(this.el, { childList: true });
  }
  async componentDidLoad() {
    debugLifecycle(`${formatEl(this.el)} did load`);
  }
  async disconnectedCallback() {
    debugLifecycle(`${formatEl(this.el)} did unload`);
    if (this.mutationObserver) {
      this.mutationObserver.disconnect();
    }
  }
  setStyle(style) {
    style = style || this.el.textContent;
    this._layer.setStyle(style);
  }
  setFilter(filter) {
    this._layer.setFilter(filter);
  }
  render() {
    return h(Host, { class: "map__layer" });
  }
  emitError(error) {
    this.flowMap._map.context.emitError(error);
  }
  static get is() { return "flow-map-layer"; }
  static get properties() { return {
    "_layer": {
      "type": "unknown",
      "mutable": true,
      "complexType": {
        "original": "FlowCoreMapLayer",
        "resolved": "FlowCoreMapLayer",
        "references": {
          "FlowCoreMapLayer": {
            "location": "import",
            "path": "../../../core/flow-core-map/flow-core-map-layer"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [{
            "name": "internal",
            "text": undefined
          }],
        "text": ""
      }
    },
    "name": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [{
            "name": "deprecated",
            "text": "Use `id` attribute instead."
          }],
        "text": "The name of the layer. Layer names should be unique for a particular map."
      },
      "attribute": "name",
      "reflect": false
    },
    "type": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The layer type."
      },
      "attribute": "type",
      "reflect": false
    },
    "source": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The name of the map data source to display."
      },
      "attribute": "source",
      "reflect": false
    },
    "sourceLayer": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The vector source layer to display. Defaults to 'geometry' for vector-type data sources.\nOmit for GeoJSON-type data sources."
      },
      "attribute": "source-layer",
      "reflect": false
    },
    "filter": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [{
            "name": "reactive",
            "text": undefined
          }],
        "text": "An optional layer filter. Specified as a Flo.w reactive expression."
      },
      "attribute": "filter",
      "reflect": false
    },
    "layerStyle": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [{
            "name": "deprecated",
            "text": "Do not use."
          }],
        "text": ""
      },
      "attribute": "layer-style",
      "reflect": false
    },
    "interactive": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Set to `true` to enable hover and click actions on the map layer."
      },
      "attribute": "interactive",
      "reflect": false,
      "defaultValue": "true"
    },
    "above": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Specify to override the default layer ordering and place this layer above the named layer."
      },
      "attribute": "above",
      "reflect": false
    },
    "below": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Specify to override the default layer ordering and place this layer below the named layer."
      },
      "attribute": "below",
      "reflect": false
    },
    "minZoom": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The minimum zoom level for the layer. Below this level, the layer will be hidden."
      },
      "attribute": "min-zoom",
      "reflect": false,
      "defaultValue": "0"
    },
    "maxZoom": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The maximum zoom level for the layer. Above this level, the layer will be hidden."
      },
      "attribute": "max-zoom",
      "reflect": false,
      "defaultValue": "18"
    }
  }; }
  static get elementRef() { return "el"; }
  static get watchers() { return [{
      "propName": "interactive",
      "methodName": "updateInteractive"
    }, {
      "propName": "minZoom",
      "methodName": "updateMinZoom"
    }, {
      "propName": "maxZoom",
      "methodName": "updateMaxZoom"
    }, {
      "propName": "layerStyle",
      "methodName": "setStyle"
    }, {
      "propName": "filter",
      "methodName": "setFilter"
    }]; }
}
__decorate([
  AncestorElement({ supportAttr: false })
], FlowMapLayer.prototype, "flowMap", void 0);
