/*!
 * (C) Emu Analytics
 */
import * as maplibreSpec from '@emuanalytics/maplibre-gl-style-spec';
import { EvaluationError } from './errors/evaluation-error';
/**
 * @internal
 */
export class MaplibreGLSpecHelper {
  constructor() {
    this.initializePropertyIndex();
  }
  static get instance() {
    if (!MaplibreGLSpecHelper._instance) {
      MaplibreGLSpecHelper._instance = new MaplibreGLSpecHelper();
    }
    return MaplibreGLSpecHelper._instance;
  }
  getPropertyMetadata(propertyKey, layerType) {
    return this.propertyIndex[layerType][propertyKey];
  }
  getPropertyType(propertyKey, layerType) {
    return this.propertyIndex[layerType][propertyKey].type;
  }
  evaluateExpression(expr) {
    const { result, value } = maplibreSpec.expression.createExpression(expr);
    if (result === 'success') {
      // !! Typescript version issues at build time
      return value.evaluate({ zoom: 0 });
    }
    // !! Typescript version issues at build time
    throw new EvaluationError(expr, value.map(v => v.message));
  }
  /**
   * Construct index of layer properties by layer type
   */
  initializePropertyIndex() {
    this.layerTypes = Object.keys(maplibreSpec.latest.layer.type.values);
    this.propertyIndex = this.layerTypes.reduce((acc, layerType) => {
      acc[layerType] = {};
      ['layout', 'paint'].forEach(propType => {
        Object.keys(maplibreSpec.latest[`${propType}_${layerType}`]).forEach(prop => {
          acc[layerType][prop] = {
            property: prop,
            mapboxProperty: prop,
            type: propType,
            definition: maplibreSpec.latest[`${propType}_${layerType}`][prop]
          };
        });
      });
      return acc;
    }, {});
  }
}
